%Model: Domany - Kinzel Cellular Automation
%       periodic boundary conditions

%The state of the site j at time t is in the variable S(t, j). S basically
%is the sheared state, so that the lattice becomes squared

clc; clear all; close all
%% PARAMETERS
mu_vec = linspace(0.75,1,10);   %Vector of values of the parameter mu
mu_vec = [mu_vec, 0.857];       %Add the critical value
Nt = 1000;                      %Total number of time steps
L = 1000;                        %Number of sites (you can consider larger systems with PBC then)
R = 10000;                       %Number of independent runs
pIC = 0.001;                     %Initial occupation probability
%% COMPUTATION
rng(1234);      %Initialization of the random number generator
Nmu = length(mu_vec);

cmap = colormap('jet');

fig_partial = figure; %Figure to store the partial results for the average
fig_density = figure; %Figure to store the time series of the density

for nmu = 1:Nmu
    mu = mu_vec(nmu);
    display(['alpha = inf - mu = ', num2str(mu)]);

    p1 = mu*tanh(1);
    p2 = mu*tanh(4);

    tic
    %Loop over independent realizations        
    N = zeros(Nt, 1); %Initialization of the variable storing the time series of the occupation for each run
    for r = 1:R
        display_progress_bar(r,R);
        S = zeros(Nt, L); %Initialization
        %Initialization of the probabilities of filling the various sites at initial time t = 1
        ptj = pIC*ones(1,L);
        S(1, :) = double(rand(1, L) < ptj); %Initial generation

        %Loop over time steps
        for t = 2:Nt
            for j = 1:L
                switch S(t-1, j) + S(t-1, mod(j, L)+1) %The mod implements PBC
                    case 2
                        S(t, j) = double(rand < p2); %Possibly occupy site in the next generation
                    case 1     
                        S(t, j) = double(rand < p1); %Possibly occupy site in the next generation
                    %case 0
                    %This is dummy because S is already 0
                    %    S(t, j) = 0; %Do not occupy a site in the next generation
                end
            end
        end

        NR(:, r) = mean(S, 2); %Store the total occupation at final time for each independent run. Sum over sites
        N = (N*(r-1) + sum(S, 2))/r; %Update the average

        if mod(r, 1000) == 0 | r == R
            figure(fig_partial)
            loglog(N/L);
            xlabel('Time t'); ylabel('Density n'); title(['Partial average over r = ', num2str(r), ' runs']);
            drawnow
        end
    end
    fprintf([' DONE in ', num2str(toc), 's\n'])

    p = polyfit(log([1:Nt])', log(N), 1);
    theta(nmu) = p(1); %Power law exponent
    

    figure(fig_density);
    p = loglog(N/L); hold on;
    ncmap = round((mu-min(mu_vec))/(max(mu_vec) - min(mu_vec))*(size(cmap,1)-1)+1); %Index of the color associated to this mu in the colormap
    p.Color = cmap(ncmap, :);
    xlabel('Time t'); ylabel('Density n');
    title(['theta = ', num2str(theta(nmu)), ' for mu = ', num2str(mu)]);
    drawnow
    savefig('alpha_inf.fig'); %Save the figure

end %End loop over mu