%Scaling analysis in the Floquet driven percolation problem
clc; clear all; close all
%% PARAMETERS
T = 20;                %Floquet period
L_vec = 100:50:500;    %Values of the number of sites considered in the scaling
Nf = 100;              %Number of Floquet steps
R = 2000;              %Number of independent runs
pF = 0.98;             %Probability of doing the Floquet flips right (that is, 1-pd, with pd the defect density used in the paper)
alpha_vec = [1.4, 18]; %Power law exponent
q = 0.9;               %Parameter of the function f (called mu in the paper)
pIC = 1;               %Initial occupation probability (called p1 in the paper)
%% COMPUTATION
rng(1234);              %Initialization of the random number generator

fig_Phi = figure;           %Figure for subharmonicity Phi vs time t, for various system sizes L
fig_scaling = figure;       %Figure for scaling of the decay time tau versus system size L
fig_phase_diagram = figure; %FIgure for the plot of beta versus alpha, that is the phase diagram

%Display messages
display('### FLOQUET SCALING ANALYSIS - invisible image')
display('Parameters: ')
display(['T = ', num2str(T)]);
display(['Nf = ', num2str(Nf)]);
display(['L  = ', num2str(L_vec)]);
display(['R  = ', num2str(R)]);
display(['PF = ', num2str(pF)]);
display(['alpha = ', num2str(alpha_vec)]);
display(['q = ', num2str(q)]);
display(['pIC = ', num2str(pIC)]);
display(['Computation starts in date ', datestr(datetime)]);

%Loop over independent realizations
%I break the independet runs in group of K runs, that are then performed
%with the parfor.
K = 20;                 %Number of independent runs that are run on the parfor
R = round(R/K)*K;       %Round R to be a multiple of K
Nr = zeros(T*Nf, R);    %Initialize variable to store the time series of the various independent runs

Nalpha = length(alpha_vec); %Number of alpha
NL = length(L_vec);         %Number of L
TAU = NaN*ones(length(alpha_vec), length(L_vec)); %Initialization of TAU.
beta = NaN*alpha_vec;       %tau = A*exp(beta*L)
A = NaN*alpha_vec;          %tau = A*exp(beta*L)
    
for nalpha = 1:Nalpha
    alpha = alpha_vec(nalpha);
    for nL = 1:NL
        L = L_vec(nL); %System size
        display(['L = ', num2str(L), ' alpha = ', num2str(alpha)]); tic

        %INITIALIZATION of the occupation probabilities p0 at initial time t = 1
        p0 = pIC*ones(1,L);     %Same probability pIC for all sites
        
        tic
        for k = 1:R/K
            display_progress_bar(k, R/K);
            parfor r = (k-1)*K+1:k*K
                Nr(:, r) = Floquet_percolation_dynamics(p0, alpha, q, L, T, Nf, pF);
            end
            r = k*K; %Number of already computed independent runs 
            N = mean(Nr(:, [1:r]), 2); %Average number of particles at final time

            
            nstroboT = N(1:T:end)/L;    %Gets the density at stroboscopic times 1, T+1, 2T+1, ...
            k = [0:Nf-2]';              %Discrete times in units of T
            Phi = (-1).^(k+1).*(nstroboT(2:end) - nstroboT(1:end-1));
        end
        fprintf([' DONE in ', num2str(toc), ' s\n'])
        
        %Exponential fit a*exp(b*x) to find the decay time.
        %Initial guesses of the parameters
        a0 = 1;
        b0 = 1;
        param0 = [a0,b0]; %Initial guess of the optimal parameters
        [param, fval] = fminsearch(@(param)ls_fun(param, k(2:end), Phi(2:end)), param0); %Do not fit the first value, which appears out of trend...
        
        TAU(nalpha, nL) = -1/param(2); %DTC lifetime
        
        %Set colors
        if alpha < 1.5
            switch nL
                case 1; col = [205, 224, 255]/255;
                case 2; col = [170, 204, 255]/255;
                case 3; col = [128, 179, 255]/255;
                case 4; col = [85, 153, 255]/255;
                case 5; col = [42, 127, 255]/255;
                case 6; col = [0, 102, 255]/255;
                case 7; col = [0, 85, 212]/255;
                case 8; col = [0, 68, 170]/255;
                case 9; col = [0, 51, 128]/255;
                otherwise; col = [0,0,0];
            end
        else
            switch nL
                case 1; col = [255, 213, 213]/255;
                case 2; col = [255, 170, 170]/255;
                case 3; col = [255, 128, 128]/255;
                case 4; col = [255, 85, 85]/255;
                case 5; col = [255, 42, 42]/255;
                case 6; col = [255, 0, 0]/255;
                case 7; col = [212, 0, 0]/255;
                case 8; col = [170, 0, 0]/255;
                case 9; col = [128, 0, 0]/255;
                otherwise; col = [0,0,0];
            end
        end
        
        %figure(fig_Phi);
        %set(0, 'CurrentFigure', fig_m);
        subplot(2, Nalpha, nalpha);
        pdata = plot(k, Phi); hold on;
        pfit  = plot(k, fit_fun(param, k), '--');
        pdata.Color = col;
        pfit.Color = col;
        title(['\alpha = ', num2str(alpha)]);
        drawnow; %ylim([0,1]);
        
        if L == max(L_vec)
            subplot(2, Nalpha, Nalpha + nalpha);
            p = plot(N/L); hold on; p.Color = pdata.Color; drawnow
        end
        
        savefig('Phi_partial.fig');
        
    end %End loop over L
    
    %Guessed functional form: tau = A*exp(beta*L) --> log(tau) = log(A) + beta*L
    p = polyfit(L_vec, log(TAU(nalpha, :)), 1);
    beta(nalpha) = p(1);
    A(nalpha) = exp(p(2));
    
    figure(fig_scaling);
    pdata = semilogy(L_vec, TAU(nalpha, :), '-x'); hold on;
    pfit = semilogy(L_vec, A(nalpha)*exp(beta(nalpha)*L_vec), '--'); pfit.Color = pdata.Color;
    xlabel('L'); ylabel('\tau'); title('semilogy'); drawnow
    
    figure(fig_phase_diagram)
    plot(alpha_vec, beta); xlabel('\alpha'); ylabel('\beta'); drawnow;
    
    format long
    display(['alpha = ', num2str(alpha)]);
    display(['beta = ', num2str(beta(nalpha))]);
    display(['A = ', num2str(A(nalpha))]);
    
    %save('scaling')
end %End loop over alpha

figure(fig_Phi);
savefig('Phi_completed.fig');

display(['Computation ends in date ', datestr(datetime)])
display(['Computation time ', num2str(toc), ' s'])

%% FUNCTIONS
%Least square function
function f = ls_fun(param, x, y)
% Least squares function
    f = sum((fit_fun(param, x) - y).^2);
end

function f = fit_fun(param, x)
% Fitting function
    a = param(1);
    b = param(2);
    f = a*exp(b*x); %Exponential
end