function [N, S] = percolation_dynamics(p0, alpha, q, L, Nt)
% p0       Initial probability distribution
% L        System size
% Nt       Number of time steps
% S        State of the system
% q        Parameter of the function f (p = f(phi)), called mu in the paper
% alpha    Exponent of the power law average

    %Initialization
    p = p0;                                                         %Occupation probabilities
    S(1, :) = double(rand(1, L) < p);                               %Occupations
    phi = order_parameter(S(1, :), alpha, NaN, NaN);    %Order parameters. The NaN are here beacuse it is the first time we compute it

    %Loop over time steps
    for t = 2:Nt
        p = q*tanh(4*phi.^2);                               %Occupation probabilities
        S(t, :) = double(rand(1, L) < p);                   %New occupations
        updated = find((2*S(t,:)-1).*(2*S(t-1,:)-1) == -1); %Indexes of the occupations that have changed in the last time step
        phi = order_parameter(S(t, :), alpha, phi, updated);%Order parameters
    end
    
    N = sum(S,2); %Time series of the total number of particles
    
    
    %FUNCTIONS
    function phi = order_parameter(s, phiold, updated)
    %Compute the order parameters of the generation s([1:L]).
    %phi(j)     order parameter for the site j.
    %phiold     order parameter at the previous time step
    %updated    list of the indexes of the sites that changed in the last step
    %NOTE: If it is the first time that this function is called, then just set
    %phiold and updated to NaN

        L = length(s);              %Number of sites

        %Kac normalization to guarantee phi in (0,1)
        Nkac = 0;               %Initialization
        for j = 1:L
            %Distance (PBC). The 1/2 is due to the triangular lattice.
            %There are two ways of computing the distance (see paper).
            %r = min(abs(1-j-1/2), L-abs(1-j-1/2));
            r = L/pi*abs(tan(pi*(1-j-1/2)/L));
            Nkac = Nkac + 1/r^alpha;
        end

        if isnan(phiold)
            %In case it is the first time, I compute it from scratch
            phi = zeros(1, L);    %Initialize
            for j = 1:L
                for i = 1:L
                    %r = min(abs(i-j-1/2), L-abs(i-j-1/2));
                    r = L/pi*abs(tan(pi*(i-j-1/2)/L));
                    phi(j) = phi(j) + s(i)/r^alpha/Nkac;
                end
            end
        else
            %Otherwise, compute phi as un update from the last computation
            for j = 1:L
                phi(j) = phiold(j); %Initialize
                for i = updated     %I only loop over the sites that have changed
                    %r = min(abs(i-j-1/2), L-abs(i-j-1/2));
                    r = L/pi*abs(tan(pi*(i-j-1/2)/L));
                    phi(j) = phi(j) + (2*s(i)-1)/r^alpha/Nkac; %Add or remove a term depending on the updated value
                end
            end
        end
    end
end

