function FPs = twoTmapFP(mu, T, toll, Nitermax, lastFPs)
% Function that finds the fixed points of the two years map.
% INPUTS
% mu        Parameter for the probability function
% T         Number of steps in the DP
% toll      Absolute tolerance in the search of the FPs
% Nitermax  Maximum number of iterations for the search
% lastFPs   Last available set of fixed points (not compulsory)
%
% OUTPUT
% FPs       Three dimensional vector with the following components
%           FPs(1) = smallest FP (either stable or NaN)
%           FPs(2) = medium FP (either unstable or stable)
%           FPs(3) = largest FP (either stable or NaN)
%           FPs(4) = 1 if tolerance is reached, 0 otherwise
%
% If the tolerance is not reached, then NaN are returned
    
    if nargin == 4
        %Default values if lastFPs is not passed
        lastFPs = [0,0.5,1];
    end
    
    %If lastFPs is passed but contains NaN, then fix them
    if isnan(lastFPs(1))
        lastFPs(1) = 0;
    end
    if isnan(lastFPs(2))
        lastFPs(2) = 0.5;
    end
    
    if isnan(lastFPs(3))
        lastFPs(3) = 1;
    end
    
    %Look for the central FP 2
    f = @(p) (FloquetMFdynamics(p, mu, T)-p);
    p2 = fzero(f, lastFPs(2));
    dp = 1e-8;
    df_dp = 1/dp*(FloquetMFdynamics(p2+dp, mu, T) - FloquetMFdynamics(p2, mu, T));
    
    if df_dp > -0.999 %Should be -1, but we allow some margin to avoid numerical errors
        %The FP is stable, and therefore the only one
        FPs = [NaN, p2, NaN, 1];
    else
        %The FP is unstable, and therefore there are other two FPs, and I
        %can find them iterating the dynamics
        
        niter = 0;
        p1_old = lastFPs(1); %Initialize
        p3_old = lastFPs(3); %Initialize
        err = Inf;            %Initialize error
        
        while err > toll & niter < Nitermax
            p1_new = FloquetMFdynamics(p3_old, mu, T);
            p3_new = FloquetMFdynamics(p1_new, mu, T);
            
            err = max(abs(p1_new - p1_old), abs(p3_new - p3_old));
            niter = niter + 1;  %Counter for the number of iterations
            
            p1_old = p1_new;    %Ready for the next iteration
            p3_old = p3_new;    %Ready for the next iteration
        end
        p1 = p1_new;
        p3 = p3_new;
        
        %Sort if not already sorted
        if p1 > p3
            c = p1;
            p1 = p3;
            p3 = c;
        end
        
        if niter == Nitermax | (p3-p1) < toll
            %If the two FPs are too close, then actually there is probably
            %just one stable FP and they coincide to p2, so we assume they
            %do not actually exist
            FPs = [NaN, p2, NaN, 0]; %Tolerance not reached
        else
            FPs = [p1, p2, p3, 1]; %Tolerance reached
        end
    end
end